@@ -43,6 +43,7 @@ warnings.filterwarnings("ignore", message="numpy.ufunc size changed")
 import mlflow.projects as projects
 import mlflow.tracking as tracking
 import mlflow.models
+import mlflow.artifacts
 
 # model flavors
 _model_flavors_supported = []
@@ -69,6 +70,7 @@ try:
     import mlflow.paddle as paddle
     import mlflow.prophet as prophet
     import mlflow.pmdarima as pmdarima
+    import mlflow.diviner as diviner
 
     _model_flavors_supported = [
         "catboost",
@@ -91,6 +93,7 @@ try:
         "paddle",
         "prophet",
         "pmdarima",
+        "diviner",
     ]
 except ImportError as e:
     # We are conditional loading these commands since the skinny client does
@@ -155,6 +158,7 @@ delete_run = mlflow.tracking.fluent.delete_run
 register_model = mlflow.tracking._model_registry.fluent.register_model
 autolog = mlflow.tracking.fluent.autolog
 evaluate = mlflow.models.evaluate
+last_active_run = mlflow.tracking.fluent.last_active_run
 
 run = projects.run
 
@@ -195,4 +199,5 @@ __all__ = [
     "list_run_infos",
     "autolog",
     "evaluate",
+    "last_active_run",
 ] + _model_flavors_supported
