@@ -15,6 +15,8 @@ import logging
 import struct
 import sys
 import math
+import urllib
+import pathlib
 from collections import OrderedDict
 from abc import ABCMeta, abstractmethod
 
@@ -107,7 +109,8 @@ class EvaluationResult:
             uri = meta["uri"]
             ArtifactCls = _get_class_from_string(meta["class_name"])
             artifact = ArtifactCls(uri=uri)
-            artifact._load(os.path.join(artifacts_dir, artifact_name))
+            filename = pathlib.Path(urllib.parse.urlparse(uri).path).name
+            artifact._load(os.path.join(artifacts_dir, filename))
             artifacts[artifact_name] = artifact
 
         return EvaluationResult(metrics=metrics, artifacts=artifacts)
@@ -131,8 +134,9 @@ class EvaluationResult:
         artifacts_dir = os.path.join(path, "artifacts")
         os.mkdir(artifacts_dir)
 
-        for artifact_name, artifact in self.artifacts.items():
-            artifact._save(os.path.join(artifacts_dir, artifact_name))
+        for artifact in self.artifacts.values():
+            filename = pathlib.Path(urllib.parse.urlparse(artifact.uri).path).name
+            artifact._save(os.path.join(artifacts_dir, filename))
 
     @property
     def metrics(self) -> Dict[str, Any]:
@@ -793,22 +797,45 @@ def evaluate(
     :param custom_metrics: (Optional) A list of custom metric functions. A custom metric
                            function is required to take in two parameters:
 
-                           - Union[pandas.Dataframe, pyspark.sql.DataFrame]: The first being a
+                           - ``Union[pandas.Dataframe, pyspark.sql.DataFrame]``: The first being a
                              Pandas or Spark DataFrame containing ``prediction`` and ``target``
                              column. The ``prediction`` column contains the predictions made by
                              the model. The ``target`` column contains the corresponding labels
                              to the predictions made on that row.
-                           - Dict: The second is a dictionary containing the metrics calculated
+                           - ``Dict``: The second is a dictionary containing the metrics calculated
                              by the default evaluator. The keys are the names of the metrics
                              and the values are the scalar values of the metrics. Refer to the
                              DefaultEvaluator behavior section for what metrics will be returned
                              based on the type of model (i.e. classifier or regressor).
+                           - (Optional) ``str``: the path to a temporary directory that can be used
+                             by the custom metric function to temporarily store produced artifacts.
+                             The directory will be deleted after the artifacts are logged.
 
                            A custom metric function can return in the following format:
 
-                           - Dict[AnyStr, Union[int, float, np.number]: a singular dictionary of
+                           - ``Dict[AnyStr, Union[int, float, np.number]``: a singular dictionary of
                              custom metrics, where the keys are the names of the metrics, and the
                              values are the scalar values of the metrics.
+                           - ``Tuple[Dict[AnyStr, Union[int,float,np.number]], Dict[AnyStr,Any]]``:
+                             a tuple of a dict containing the custom metrics, and a dict of
+                             artifacts, where the keys are the names of the artifacts, and the
+                             values are objects representing the artifacts.
+
+                           Object types that artifacts can be represented as:
+
+                           - A string uri representing the file path to the artifact. MLflow will
+                             infer the type of the artifact based on the file extension.
+                           - A string representation of a JSON object. This will be saved as a
+                             .json artifact.
+                           - Pandas DataFrame. This will be resolved as a CSV artifact.
+                           - Numpy array. This will be saved as a .npy artifact.
+                           - Matplotlib Figure. This will be saved as an image artifact. Note that
+                             ``matplotlib.pyplot.savefig`` is called behind the scene with default
+                             configurations. To customize, either save the figure with the desired
+                             configurations and return its file path or define customizations
+                             through environment variables in ``matplotlib.rcParams``.
+                           - Other objects will be attempted to be pickled with the default
+                             protocol.
 
                            .. code-block:: python
                                :caption: Custom Metric Function Boilerplate
@@ -816,29 +843,48 @@ def evaluate(
                                def custom_metrics_boilerplate(eval_df, builtin_metrics):
                                    # ...
                                    metrics: Dict[AnyStr, Union[int, float, np.number]] = some_dict
+                                   artifacts: Dict[AnyStr, Any] = some_artifact_dict
                                    # ...
+                                   if artifacts is not None:
+                                       return metrics, artifacts
                                    return metrics
 
                            .. code-block:: python
                                :caption: Example usage of custom metrics
 
                                def squared_diff_plus_one(eval_df, builtin_metrics):
-                                 return {
-                                     "squared_diff_plus_one": (
-                                         np.sum(
-                                             np.abs(
-                                                 eval_df["prediction"] - eval_df["target"] + 1
-                                             ) ** 2
-                                         )
-                                     )
-                                 }
+                                   return {
+                                       "squared_diff_plus_one": (
+                                           np.sum(
+                                               np.abs(
+                                                   eval_df["prediction"] - eval_df["target"] + 1
+                                               ) ** 2
+                                           )
+                                       )
+                                   }
+
+                               def scatter_plot(eval_df, builtin_metrics, artifacts_dir):
+                                   import tempfile
+                                   plt.scatter(eval_df['prediction'], eval_df['target'])
+                                   plt.xlabel('Targets')
+                                   plt.ylabel('Predictions')
+                                   plt.title("Targets vs. Predictions")
+                                   plt.savefig(os.path.join(artifacts_dir, "example.png"))
+                                   return {}, {
+                                       "pred_target_scatter": os.path.join(
+                                            artifacts_dir, "example.png"
+                                       )
+                                   }
 
                                with mlflow.start_run():
                                    mlflow.evaluate(
                                        model,
-                                       X,
+                                       data,
                                        targets,
-                                       custom_metrics=[squared_diff_plus_one, ...],
+                                       model_type,
+                                       dataset_name,
+                                       evaluators,
+                                       custom_metrics=[squared_diff_plus_one, scatter_plot],
                                    )
 
     :return: An :py:class:`mlflow.models.EvaluationResult` instance containing
