@@ -237,7 +237,7 @@ from mlflow.pyfunc.model import get_default_pip_requirements
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.types import DataType, Schema, TensorSpec
 from mlflow.types.utils import clean_tensor_type
-from mlflow.utils import PYTHON_VERSION, get_major_minor_py_version
+from mlflow.utils import PYTHON_VERSION, get_major_minor_py_version, _is_in_ipython_notebook
 from mlflow.utils.annotations import deprecated
 from mlflow.utils.file_utils import _copy_file_or_tree, write_to
 from mlflow.utils.model_utils import (
@@ -249,14 +249,16 @@ from mlflow.utils.model_utils import (
 )
 from mlflow.utils.uri import append_to_uri_path
 from mlflow.utils.environment import (
-    _EnvManager,
     _validate_env_arguments,
     _process_pip_requirements,
     _process_conda_env,
     _CONDA_ENV_FILE_NAME,
     _REQUIREMENTS_FILE_NAME,
     _CONSTRAINTS_FILE_NAME,
+    _PYTHON_ENV_FILE_NAME,
+    _PythonEnv,
 )
+from mlflow.utils import env_manager as _EnvManager
 from mlflow.utils.docstring_utils import format_docstring, LOG_MODEL_PARAM_DOCS
 from mlflow.utils.databricks_utils import is_in_databricks_runtime
 from mlflow.utils.file_utils import get_or_create_tmp_dir, get_or_create_nfs_tmp_dir
@@ -806,10 +808,11 @@ def get_model_dependencies(model_uri, format="pip"):  # pylint: disable=redefine
              specifying the model's dependencies.
     """
     dep_file = _get_model_dependencies(model_uri, format)
+
     if format == "pip":
-        prefix = "%" if is_in_databricks_runtime() else ""
+        prefix = "%" if _is_in_ipython_notebook() else ""
         _logger.info(
-            "To install these model dependencies, run the "
+            "To install the dependencies that were used to train the model, run the "
             f"following command: '{prefix}pip install -r {dep_file}'."
         )
     return dep_file
@@ -967,14 +970,16 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
 
         - ``ArrayType(StringType)``: All columns converted to ``string``.
 
-    :param env_manager: The environment manager to use in order to create the
-                        software environment for model inference. Default value is ``local``,
-                        The following values are supported:
+    :param env_manager: The environment manager to use in order to create the python environment
+                        for model inference. Note that environment is only restored in the context
+                        of the PySpark UDF; the software environment outside of the UDF is
+                        unaffected. Default value is ``local``, and the following values are
+                        supported:
 
                          - ``conda``: (Recommended) Use Conda to restore the software environment
-                           that was used to train the model. Note that environment is only restored
-                           in the context of the PySpark UDF; the software environment outside of
-                           the UDF is unaffected.
+                           that was used to train the model.
+                         - ``virtualenv``: Use virtualenv to restore the python environment that
+                           was used to train the model.
                          - ``local``: Use the current Python environment for model inference, which
                            may differ from the environment used to train the model and may lead to
                            errors or invalid predictions.
@@ -998,7 +1003,7 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
     from pyspark.sql.types import DoubleType, IntegerType, FloatType, LongType, StringType
     from mlflow.models.cli import _get_flavor_backend
 
-    env_manager = _EnvManager.from_string(env_manager)
+    _EnvManager.validate(env_manager)
 
     # Check whether spark is in local or local-cluster mode
     # this case all executors and driver share the same filesystem
@@ -1007,7 +1012,7 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
     nfs_root_dir = get_nfs_cache_root_dir()
     should_use_nfs = nfs_root_dir is not None
     should_use_spark_to_broadcast_file = not (is_spark_in_local_mode or should_use_nfs)
-    conda_env_root_dir = _get_or_create_env_root_dir(should_use_nfs)
+    env_root_dir = _get_or_create_env_root_dir(should_use_nfs)
 
     if not isinstance(result_type, SparkDataType):
         result_type = _parse_datatype_string(result_type)
@@ -1029,7 +1034,7 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
         artifact_uri=model_uri, output_path=_create_model_downloading_tmp_dir(should_use_nfs)
     )
 
-    if env_manager is _EnvManager.LOCAL:
+    if env_manager == _EnvManager.LOCAL:
         # Assume spark executor python environment is the same with spark driver side.
         _warn_dependency_requirement_mismatches(local_model_path)
         _logger.warning(
@@ -1065,12 +1070,12 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
         # "capture_output=False" is the output will be printed immediately, otherwise you have
         # to wait conda command fail and suddenly get all output printed (included in error
         # message).
-        if env_manager is _EnvManager.CONDA:
+        if env_manager != _EnvManager.LOCAL:
             _get_flavor_backend(
                 local_model_path,
-                env_manager=_EnvManager.CONDA,
+                env_manager=env_manager,
                 install_mlflow=False,
-                env_root_dir=conda_env_root_dir,
+                env_root_dir=env_root_dir,
             ).prepare_env(model_uri=local_model_path, capture_output=is_in_databricks_runtime())
 
     # Broadcast local model directory to remote worker if needed.
@@ -1160,24 +1165,23 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
 
         scoring_server_proc = None
 
-        # TODO: Support virtual env.
-        if env_manager is _EnvManager.CONDA:
+        if env_manager != _EnvManager.LOCAL:
             if should_use_spark_to_broadcast_file:
                 local_model_path_on_executor = _SparkDirectoryDistributor.get_or_extract(
                     archive_path
                 )
                 # Create individual conda_env_root_dir for each spark UDF task process.
-                conda_env_root_dir_on_executor = _get_or_create_env_root_dir(should_use_nfs)
+                env_root_dir_on_executor = _get_or_create_env_root_dir(should_use_nfs)
             else:
                 local_model_path_on_executor = local_model_path
-                conda_env_root_dir_on_executor = conda_env_root_dir
+                env_root_dir_on_executor = env_root_dir
 
             pyfunc_backend = _get_flavor_backend(
                 local_model_path_on_executor,
                 workers=1,
                 install_mlflow=False,
-                env_manager=_EnvManager.CONDA,
-                env_root_dir=conda_env_root_dir_on_executor,
+                env_manager=env_manager,
+                env_root_dir=env_root_dir_on_executor,
             )
 
             if should_use_spark_to_broadcast_file:
@@ -1194,12 +1198,12 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
                 )
 
             # launch scoring server
-            # TODO: adjust timeout for server requests handler.
             server_port = find_free_port()
             scoring_server_proc = pyfunc_backend.serve(
                 model_uri=local_model_path_on_executor,
                 port=server_port,
                 host="127.0.0.1",
+                timeout=60,
                 enable_mlserver=False,
                 synchronous=False,
                 stdout=subprocess.PIPE,
@@ -1242,7 +1246,7 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
             def batch_predict_fn(pdf):
                 return client.invoke(pdf)
 
-        elif env_manager is _EnvManager.LOCAL:
+        elif env_manager == _EnvManager.LOCAL:
             if should_use_spark_to_broadcast_file:
                 loaded_model, _ = SparkModelCache.get_or_load(archive_path)
             else:
@@ -1671,6 +1675,8 @@ def _save_model_with_loader_module_and_data_path(
 
     # Save `requirements.txt`
     write_to(os.path.join(path, _REQUIREMENTS_FILE_NAME), "\n".join(pip_requirements))
+
+    _PythonEnv.current().to_yaml(os.path.join(path, _PYTHON_ENV_FILE_NAME))
     return mlflow_model
 
 
