@@ -15,6 +15,7 @@ from collections import namedtuple
 import logging
 import re
 from typing import NamedTuple, Optional
+from pathlib import Path
 
 import mlflow
 from mlflow.exceptions import MlflowException
@@ -75,7 +76,7 @@ def _join_continued_lines(lines):
 _Requirement = namedtuple("_Requirement", ["req_str", "is_constraint"])
 
 
-def _parse_requirements(requirements, is_constraint):
+def _parse_requirements(requirements, is_constraint, base_dir=None):
     """
     A simplified version of `pip._internal.req.parse_requirements` which performs the following
     operations on the given requirements file and yields the parsed requirements.
@@ -88,6 +89,8 @@ def _parse_requirements(requirements, is_constraint):
     :param requirements: A string path to a requirements file on the local filesystem or
                          an iterable of pip requirement strings.
     :param is_constraint: Indicates the parsed requirements file is a constraint file.
+    :param base_dir: If specified, resolve relave file references (e.g. '-r requirements.txt')
+                     against the specified directory.
     :return: A list of ``_Requirement`` instances.
 
     References:
@@ -98,12 +101,13 @@ def _parse_requirements(requirements, is_constraint):
     - Constraints Files:
       https://pip.pypa.io/en/stable/user_guide/#constraints-files
     """
-    if isinstance(requirements, str):
-        base_dir = os.path.dirname(requirements)
-        with open(requirements) as f:
-            requirements = f.read().splitlines()
-    else:
-        base_dir = os.getcwd()
+    if base_dir is None:
+        if isinstance(requirements, (str, Path)):
+            base_dir = os.path.dirname(requirements)
+            with open(requirements) as f:
+                requirements = f.read().splitlines()
+        else:
+            base_dir = os.getcwd()
 
     lines = map(str.strip, requirements)
     lines = map(_strip_inline_comment, lines)
@@ -130,6 +134,15 @@ def _flatten(iterable):
     return chain.from_iterable(iterable)
 
 
+# https://www.python.org/dev/peps/pep-0508/#names
+_PACKAGE_NAME_REGEX = re.compile(r"^(\w+|\w+[\w._-]*\w+)")
+
+
+def _get_package_name(requirement):
+    m = _PACKAGE_NAME_REGEX.match(requirement)
+    return m and m.group(1)
+
+
 _NORMALIZE_REGEX = re.compile(r"[-_.]+")
 
 
